#include "pch.hpp"
#include "chalcopyrite.hpp"
#include "mainwnd.hpp"
#include "scorelinedlg.hpp"
#include "scoreline.hpp"
#include "Formatter.hpp"
#include "characters.hpp"
#include "SwrAddrDef.h"
#include "resource.h"

#define MINIMAL_USE_PROCESSHEAPSTRING
#include "MinimalPath.hpp"

#define UC_OPENHIT 0xDEAD

#define EVEN_COLOR 0xFFFFFF
#define ODD_COLOR  0xD0DCD0

static struct {
	DWORD fmt;
	int cx;
	LPCSTR text;
} s_listColumns[] = {
	{ LVCFMT_LEFT,   80, "O" },
	{ LVCFMT_RIGHT,  55, "ΐ퐔" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "ߋ30" },
};

static HMENU s_hSysMenu;

static void SysMenu_OnClose(HWND hDlg, int x, int y)
{
	ShowWindow(hDlg, SW_HIDE);
}

static LRESULT ScoreLineView_OnCustomDraw(HWND hParent, HWND hwnd, LPNMLVCUSTOMDRAW lpnlvCustomDraw)
{
	switch(lpnlvCustomDraw->nmcd.dwDrawStage) {
	case CDDS_PREPAINT:
		return CDRF_NOTIFYITEMDRAW;
	case CDDS_ITEMPREPAINT:
		if(lpnlvCustomDraw->nmcd.dwItemSpec & 1)
			lpnlvCustomDraw->clrTextBk = EVEN_COLOR;
		else
			lpnlvCustomDraw->clrTextBk = ODD_COLOR;
		return CDRF_NOTIFYSUBITEMDRAW;
	default:
		return CDRF_DODEFAULT;
	}
}

static LRESULT ScoreLIneView_OnDoubleClick(HWND hParent, HWND hwnd)
{
	int index = ListView_GetSelectionMark(hwnd);
	if(index < SWRCHAR_MAX) {
		HWND hSpecDlg = CreateDialogParam(
			g_hInstance,
			MAKEINTRESOURCE(IDD_SCORELINE_QIB),
			hParent,
			ScoreLineQIBSpecDialog_DlgProc,
			index);
		ShowWindow(hSpecDlg, SW_SHOW);
	}
	return TRUE;
}

__inline
static void ScoreLineQIBDialog_RefleshUnit(HWND listWnd, int i, int win, int lose)
{
	Minimal::ProcessHeapStringT<char> itemstr;

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;

	int sum = win + lose;
	// ΐ퐔
	itemstr = Formatter("%d", sum);
	item.iItem = i;
	item.iSubItem = 1;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = Formatter("%d", win);
	item.iSubItem = 2;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = Formatter("%d", lose);
	item.iSubItem = 3;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = 
		Formatter("%d%%", 
			::MulDiv(win, 100, sum ? sum : 1));
	item.iSubItem = 4;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
}

static void ScoreLineQIBDialog_Reflesh(HWND hDlg)
{
	Minimal::ProcessHeapStringT<char> baseName;
	baseName = ScoreLine_GetPath();
	::PathStripPath(baseName);
	::PathRemoveExtension(baseName);
	baseName.Repair();
	::SetWindowText(hDlg,
		Formatter(WINDOW_TEXT " - %s",
			static_cast<LPCSTR>(baseName)));

	ScoreLine_SetReadMode(SCORELINE_READALL);
	HWND listWnd = GetDlgItem(hDlg, IDC_LIST_SCORELINE);

	int sumWinAll, sumLoseAll;
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int sumWin, sumLose;
		sumWin = sumLose = 0;
		for(int j = 0; j < SWRCHAR_MAX; ++j) {
			sumWin  += ScoreLine_Read(i, j, 0);
			sumLose += ScoreLine_Read(i, j, 1);
		}
		sumWinAll  += sumWin;
		sumLoseAll += sumLose;
		ScoreLineQIBDialog_RefleshUnit(listWnd, i, sumWin, sumLose);
	}
	ScoreLineQIBDialog_RefleshUnit(listWnd, SWRCHAR_MAX + 1, sumWinAll, sumLoseAll);

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;

	Minimal::ProcessHeapStringT<char> itemstr;
	// ߋ30폟()
	ScoreLine_SetReadMode(SCORELINE_READL30S);
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int sumWin, sumLose, sum;
		sumWin = sumLose = 0;
		for(int j = 0; j < SWRCHAR_MAX; ++j) {
			sumWin  += ScoreLine_Read(i, j, 0);
			sumLose += ScoreLine_Read(i, j, 1);
		}
		sumWinAll  += sumWin;
		sumLoseAll += sumLose;
		sum = sumWin + sumLose;
		itemstr = 
			Formatter("(%d%%)", 
				::MulDiv(sumWin, 100, sum ? sum : 1));
		item.iItem = i;
		item.iSubItem = 5;
		item.pszText = (LPSTR)itemstr;
		ListView_SetItem(listWnd, &item);
	}
	// ߋ30폟
	ScoreLine_SetReadMode(SCORELINE_READL30);
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int sumWin, sumLose;
		sumWin = sumLose = 0;
		for(int j = 0; j < SWRCHAR_MAX; ++j) {
			sumWin  += ScoreLine_Read(i, j, 0);
			sumLose += ScoreLine_Read(i, j, 1);
		}
		sumWinAll  += sumWin;
		sumLoseAll += sumLose;
	}
	ScoreLineQIBDialog_RefleshUnit(listWnd, SWRCHAR_MAX + 2, sumWinAll, sumLoseAll);

	// ߋ50폟
	ScoreLine_SetReadMode(SCORELINE_READL50);
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int sumWin, sumLose;
		sumWin = sumLose = 0;
		for(int j = 0; j < SWRCHAR_MAX; ++j) {
			sumWin  += ScoreLine_Read(i, j, 0);
			sumLose += ScoreLine_Read(i, j, 1);
		}
		sumWinAll  += sumWin;
		sumLoseAll += sumLose;
	}
	ScoreLineQIBDialog_RefleshUnit(listWnd, SWRCHAR_MAX + 3, sumWinAll, sumLoseAll);

	// ߋ100폟
	ScoreLine_SetReadMode(SCORELINE_READL100);
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int sumWin, sumLose;
		sumWin = sumLose = 0;
		for(int j = 0; j < SWRCHAR_MAX; ++j) {
			sumWin  += ScoreLine_Read(i, j, 0);
			sumLose += ScoreLine_Read(i, j, 1);
		}
		sumWinAll  += sumWin;
		sumLoseAll += sumLose;
	}
	ScoreLineQIBDialog_RefleshUnit(listWnd, SWRCHAR_MAX + 4, sumWinAll, sumLoseAll);
}

static BOOL ScoreLineQIBDialog_InitListView(HWND hDlg)
{
	HWND listWnd = GetDlgItem(hDlg, IDC_LIST_SCORELINE);
	ListView_SetExtendedListViewStyle(listWnd, LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);

	LVCOLUMN column;
	column.mask	= LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
	for(int i = 0; i < ARRAYSIZE(s_listColumns); ++i) {
		column.fmt = s_listColumns[i].fmt;
		column.cx = s_listColumns[i].cx;
		column.cchTextMax = 256;
		column.pszText = (LPSTR)s_listColumns[i].text;
		ListView_InsertColumn(listWnd, i, &column);
	}

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;
	item.iSubItem = 0;
	for(int i = 0; i < ARRAYSIZE(g_characters); ++i) {
		item.pszText = (LPSTR)g_characters[i].abbr;
		item.iItem = i;
		ListView_InsertItem(listWnd, &item);
	}
	item.mask = 0; item.iItem++;
	ListView_InsertItem(listWnd, &item);

	item.mask = LVIF_TEXT; item.iItem++;
	item.pszText = "";
	ListView_InsertItem(listWnd, &item);

	item.pszText = "ߋ30"; item.iItem++; 
	ListView_InsertItem(listWnd, &item);
	item.pszText = "ߋ50"; item.iItem++; 
	ListView_InsertItem(listWnd, &item);
	item.pszText = "ߋ100"; item.iItem++; 
	ListView_InsertItem(listWnd, &item);

	return TRUE;
}

static void SysMenu_OnOpenProfile(HWND hDlg, int x, int y)
{
	char fileName[1025];
	OPENFILENAME ofn;
	::ZeroMemory(&ofn, sizeof ofn);
	ofn.lStructSize = sizeof ofn;
	ofn.hwndOwner = hDlg;
	ofn.lpstrFile = fileName;
	ofn.nMaxFile = sizeof fileName;
	ofn.lpstrDefExt = "hit";
	ofn.lpstrFilter = "Hikouseki Trackrecord (*.hit)\0*.hit\0";
	ofn.Flags = OFN_CREATEPROMPT | OFN_NOCHANGEDIR;
	fileName[0] = 0;
	if(::GetOpenFileName(&ofn)) {
		if(ScoreLine_Map(fileName)) {
			ScoreLineQIBDialog_Reflesh(hDlg);
		} else {
			MessageBox(hDlg, "vt@C̃}bsOɎs܂B", NULL, MB_OK | MB_ICONSTOP);
		}
	}
}

static void ScoreLineQIBDialog_InitSysMenu(HWND hDlg)
{
	int itemIndex = 0;
	::InsertMenu(s_hSysMenu, itemIndex++, MF_STRING | MF_BYPOSITION, UC_OPENHIT, "vt@Cǂݍ");
	::InsertMenu(s_hSysMenu, itemIndex++, MF_SEPARATOR | MF_BYPOSITION, 0, NULL);
}

static BOOL ScoreLineQIBDialog_OnInitDialog(HWND hDlg, HWND hwndFocus, LPARAM lParam)
{
	s_hSysMenu = ::GetSystemMenu(hDlg, FALSE);
	if(s_hSysMenu == NULL) return FALSE;

	int x, y;
	x = g_settings.ReadInteger("QIBDlg.Display", "X", CW_USEDEFAULT);
	y = g_settings.ReadInteger("QIBDlg.Display", "Y", CW_USEDEFAULT);
	if(x != CW_USEDEFAULT && y != CW_USEDEFAULT)
		SetWindowPos(hDlg, 0, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);

	ScoreLineQIBDialog_InitSysMenu(hDlg);
	ScoreLineQIBDialog_InitListView(hDlg);
	ScoreLineQIBDialog_Reflesh(hDlg);
	return TRUE;
}

static void ScoreLineQIBDialog_OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
}

static void ScoreLineQIBDialog_OnSysCommand(HWND hDlg, UINT nID, int x, int y)
{
	if(nID == SC_CLOSE) {
		SysMenu_OnClose(hDlg, x, y);
	} else if(nID == UC_OPENHIT) {
		SysMenu_OnOpenProfile(hDlg, x, y);
	}
}

static LRESULT ScoreLineQIBDialog_OnNotify(HWND hDlg, int idCtrl, LPNMHDR pNMHdr)
{
	switch(idCtrl) {
	case IDC_LIST_SCORELINE:
		switch(pNMHdr->code) {
		case NM_CUSTOMDRAW:
			return ScoreLineView_OnCustomDraw(hDlg, pNMHdr->hwndFrom, (LPNMLVCUSTOMDRAW)pNMHdr);
		case NM_DBLCLK:
			return ScoreLIneView_OnDoubleClick(hDlg, pNMHdr->hwndFrom);
		}
		break;
	}
	return FALSE;
}

static void ScoreLineQIBDialog_OnDestroy(HWND hDlg)
{
	RECT rct;
	GetWindowRect(hDlg, &rct);
	g_settings.WriteInteger("QIBDlg.Display", "X", rct.left);
	g_settings.WriteInteger("QIBDlg.Display", "Y", rct.top);

}

static void ScoreLineQIBDialog_OnUpdateScoreLine(HWND hDlg)
{
	ScoreLineQIBDialog_Reflesh(hDlg);
}

BOOL CALLBACK ScoreLineQIBDialog_DlgProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg) {
	HANDLE_DLG_MSG(hDlg, WM_INITDIALOG, ScoreLineQIBDialog_OnInitDialog);
	HANDLE_DLG_MSG(hDlg, WM_DESTROY, ScoreLineQIBDialog_OnDestroy);
	HANDLE_DLG_MSG(hDlg, WM_COMMAND, ScoreLineQIBDialog_OnCommand);
	HANDLE_DLG_MSG(hDlg, WM_SYSCOMMAND, ScoreLineQIBDialog_OnSysCommand);
	HANDLE_DLG_MSG(hDlg, WM_NOTIFY, ScoreLineQIBDialog_OnNotify);
	case UM_UPDATESCORELINE: ScoreLineQIBDialog_OnUpdateScoreLine(hDlg); break;
	}
	return FALSE;
}
