#include "pch.hpp"
#include "scorelinedlg.hpp"
#include "scoreline.hpp"
#include "Formatter.hpp"
#include "characters.hpp"
#include "SwrAddrDef.h"
#include "resource.h"

#define MINIMAL_USE_PROCESSHEAPSTRING
#include "MinimalPath.hpp"

#define EVEN_COLOR 0xFFFFFF
#define ODD_COLOR  0xD0DCD0

static struct {
	DWORD fmt;
	int cx;
	LPCSTR text;
} s_listColumns[] = {
	{ LVCFMT_LEFT,   80, "O" },
	{ LVCFMT_RIGHT,  55, "ΐ퐔" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "" },
	{ LVCFMT_RIGHT,  55, "ߋ30" },
};

static void SysMenu_OnClose(HWND hDlg, int x, int y)
{
	DestroyWindow(hDlg);
}

static LRESULT ScoreLineView_OnCustomDraw(HWND hwnd, LPNMLVCUSTOMDRAW lpnlvCustomDraw)
{
	switch(lpnlvCustomDraw->nmcd.dwDrawStage) {
	case CDDS_PREPAINT:
		return CDRF_NOTIFYITEMDRAW;
	case CDDS_ITEMPREPAINT:
		if(lpnlvCustomDraw->nmcd.dwItemSpec & 1)
			lpnlvCustomDraw->clrTextBk = EVEN_COLOR;
		else
			lpnlvCustomDraw->clrTextBk = ODD_COLOR;
		return CDRF_NOTIFYSUBITEMDRAW;
	default:
		return CDRF_DODEFAULT;
	}
}

__inline
static void ScoreLineQIBSpecDialog_RefleshUnit(HWND listWnd, int i, int win, int lose)
{
	Minimal::ProcessHeapStringT<char> itemstr;

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;

	int sum = win + lose;
	// ΐ퐔
	itemstr = Formatter("%d", sum);
	item.iItem = i;
	item.iSubItem = 1;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = Formatter("%d", win);
	item.iSubItem = 2;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = Formatter("%d", lose);
	item.iSubItem = 3;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
	// 
	itemstr = 
		Formatter("%d%%", 
			::MulDiv(win, 100, sum ? sum : 1));
	item.iSubItem = 4;
	item.pszText = (LPSTR)itemstr;
	ListView_SetItem(listWnd, &item);
}

static void ScoreLineQIBSpecDialog_Reflesh(HWND hDlg, int index)
{
	ScoreLine_SetReadMode(SCORELINE_READALL);
	HWND listWnd = GetDlgItem(hDlg, IDC_LIST_SCORELINE);

	int sumWinAll, sumLoseAll;
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		ScoreLineQIBSpecDialog_RefleshUnit(listWnd, i, 
			ScoreLine_Read(index, i, 0), ScoreLine_Read(index, i, 1));
	}

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;

	Minimal::ProcessHeapStringT<char> itemstr;
	// ߋ30폟
	ScoreLine_SetReadMode(SCORELINE_READL30S);
	sumWinAll = sumLoseAll = 0;
	for(int i = 0; i < SWRCHAR_MAX; ++i) {
		int win, lose, sum;
		win  = ScoreLine_Read(index, i, 0);
		lose = ScoreLine_Read(index, i, 1);
		sum  = win + lose;
		itemstr = 
			Formatter("(%d%%)", 
				::MulDiv(win, 100, sum ? sum : 1));
		item.iItem = i;
		item.iSubItem = 5;
		item.pszText = (LPSTR)itemstr;
		ListView_SetItem(listWnd, &item);
	}
}

static BOOL ScoreLineQIBSpecDialog_InitListView(HWND hDlg)
{
	HWND listWnd = GetDlgItem(hDlg, IDC_LIST_SCORELINE);
	ListView_SetExtendedListViewStyle(listWnd, LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT);

	LVCOLUMN column;
	column.mask	= LVCF_FMT | LVCF_TEXT | LVCF_WIDTH;
	for(int i = 0; i < ARRAYSIZE(s_listColumns); ++i) {
		column.fmt = s_listColumns[i].fmt;
		column.cx = s_listColumns[i].cx;
		column.cchTextMax = 256;
		column.pszText = (LPSTR)s_listColumns[i].text;
		ListView_InsertColumn(listWnd, i, &column);
	}

	LVITEM item;
	item.mask = LVIF_TEXT;
	item.cchTextMax = 256;
	item.iSubItem = 0;
	for(int i = 0; i < ARRAYSIZE(g_characters); ++i) {
		item.pszText = (LPSTR)g_characters[i].abbr;
		item.iItem = i;
		ListView_InsertItem(listWnd, &item);
	}

	return TRUE;
}

static BOOL ScoreLineQIBSpecDialog_OnInitDialog(HWND hDlg, HWND hwndFocus, LPARAM lParam)
{
	::SetWindowText(hDlg,
		Formatter("ڍ - %s",
			g_characters[lParam].abbr));

	ScoreLineQIBSpecDialog_InitListView(hDlg);
	ScoreLineQIBSpecDialog_Reflesh(hDlg, lParam);
	return TRUE;
}


static void ScoreLineQIBSpecDialog_OnCommand(HWND hDlg, int id, HWND hwndCtl, UINT codeNotify)
{
}

static void ScoreLineQIBSpecDialog_OnSysCommand(HWND hDlg, UINT nID, int x, int y)
{
	if(nID == SC_CLOSE) {
		SysMenu_OnClose(hDlg, x, y);
	}
}

static LRESULT ScoreLineQIBSpecDialog_OnNotify(HWND hDlg, int idCtrl, LPNMHDR pNMHdr)
{
	switch(idCtrl) {
	case IDC_LIST_SCORELINE:
		switch(pNMHdr->code) {
		case NM_CUSTOMDRAW:
			return ScoreLineView_OnCustomDraw(pNMHdr->hwndFrom, (LPNMLVCUSTOMDRAW)pNMHdr);
		}
		break;
	}
	return FALSE;
}

static void ScoreLineQIBSpecDialog_OnDestroy(HWND hDlg)
{
}

BOOL CALLBACK ScoreLineQIBSpecDialog_DlgProc(HWND hDlg, UINT Msg, WPARAM wParam, LPARAM lParam)
{
	switch(Msg) {
	HANDLE_DLG_MSG(hDlg, WM_INITDIALOG, ScoreLineQIBSpecDialog_OnInitDialog);
	HANDLE_DLG_MSG(hDlg, WM_DESTROY, ScoreLineQIBSpecDialog_OnDestroy);
	HANDLE_DLG_MSG(hDlg, WM_COMMAND, ScoreLineQIBSpecDialog_OnCommand);
	HANDLE_DLG_MSG(hDlg, WM_SYSCOMMAND, ScoreLineQIBSpecDialog_OnSysCommand);
	HANDLE_DLG_MSG(hDlg, WM_NOTIFY, ScoreLineQIBSpecDialog_OnNotify);
	}
	return FALSE;
}
